@@ -99,8 +99,8 @@ pxweb_as_data_frame.pxweb_levels <- function(x, optional = FALSE, ..., stringsAs
   
   df <- list()
   for(i in seq_along(x)){
-    df[[i]] <- as.data.frame(x[[i]], optional = optional, ..., stringsAsFactors = FALSE)
-    if(x[[i]]$type == "l") {
+    df[[i]] <- as.data.frame(x[[i]], optional = optional, stringsAsFactors = FALSE)
+    if(x[[i]]$type %in% c("l", "h")) {
       df[[i]]$updated <- NA
     }
   }
